==> ordering-core-1.5.2-beta.0.tgz: package/lib/esm/auth.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios, { isAxiosError } from 'axios';
import { handleAPIError } from './apiTools';
import { Mutex } from './utils/mutex';
import { jwtDecode } from 'jwt-decode';
let _authData = null;
const mutex = new Mutex(250);
export function authorizeWithUserPass(baseUrl, tenant, basicAuthPass, username, password) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            response = yield axios.post(`${baseUrl}/auth-oauth2/oauth/token`, { username, password, grant_type: 'password', scope: 'read' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authData = response.data;
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error);
            return false;
        }
    });
}
const invalidRefreshTokens = new Map();
export function authorizeWithRefreshToken(baseUrl, tenant, basicAuthPass, refreshToken) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let response = null;
        try {
            response = yield axios.post(`${baseUrl}/auth-oauth2/oauth/token`, { refresh_token: refreshToken, grant_type: 'refresh_token' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authData = response.data;
            // console.log('authData', _authData);
            //console.log('REFRESH TOKEN AUTH RESULT ***');
            //console.log(_authData);
            return true;
        }
        catch (error) {
            if (isAxiosError(error) && /^4\d{2}$/.test(`${(_a = error.response) === null || _a === void 0 ? void 0 : _a.status}`)) {
                invalidRefreshTokens.set(refreshToken, Date.now());
            }
            //console.error('Authorization error');
            // console.error(error);
            return false;
        }
    });
}
export function isTokenExpired(token) {
    // check if we can use current token
    if (token) {
        try {
            return (
            // 10s before actual expiry
            jwtDecode(token).exp * 1000 < new Date().getTime() + 10000);
        }
        catch (error) {
            // Not a JWT token
            return true;
        }
    }
    return true;
}
function isJwtToken(token) {
    try {
        jwtDecode(token);
        return true;
    }
    catch (error) {
        return false;
    }
}
export const authDataProvider = (ctx_1, refreshTokenStorageHandler_1, ...args_1) => __awaiter(void 0, [ctx_1, refreshTokenStorageHandler_1, ...args_1], void 0, function* (ctx, refreshTokenStorageHandler, forceRefresh = false) {
    // console.log( `authDataProvider invoked ----     currentToken=${  _authData ? _authData.access_token : '(null)'   }`, );
    if (!forceRefresh && _authData && _authData.access_token) {
        if (!isTokenExpired(_authData.access_token)) {
            return { token: _authData.access_token, UUID: _authData.UUID };
        }
    }
    //console.log('******** GET AUTH TOKEN (REMOTE CALL) ********');
    let loginSuccess = false;
    const refreshToken = yield refreshTokenStorageHandler.getRefreshToken(ctx.TENANT);
    if (refreshToken &&
        (!isJwtToken(refreshToken) || !isTokenExpired(refreshToken))) {
        //console.log('*** AUTHORIZE WITH REFRESH TOKEN ');
        // use mutex to enforce max 1 api call at a time
        loginSuccess = yield mutex.getLock(() => {
            console.log('checking token', refreshToken);
            if (_authData && !isTokenExpired(_authData.access_token)) {
                return Promise.resolve(true);
            }
            // If same refresh token was invalid less then 30s ago skip api call
            const invalidTokenTimestamp = invalidRefreshTokens.get(refreshToken);
            console.log('invalidTokenTimestamp', invalidTokenTimestamp, Date.now());
            if (invalidTokenTimestamp && Date.now() < invalidTokenTimestamp + 30000) {
                return Promise.resolve(false);
            }
            return authorizeWithRefreshToken(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, refreshToken);
        });
    }
    if (!loginSuccess && ctx.anonymousAuth) {
        // console.log('**** ANONYMOUS AUTH ****');
        loginSuccess = yield authorizeWithUserPass(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, 'anonymous', '');
    }
    if (!loginSuccess || !_authData) {
        refreshTokenStorageHandler.clearRefreshToken(ctx.TENANT);
        return { token: '', UUID: '' };
    }
    else {
        refreshTokenStorageHandler.setRefreshToken(ctx.TENANT, _authData.refresh_token);
        return { token: _authData.access_token, UUID: _authData.UUID };
    }
});
export function createAuthDataProvider(ctx, refreshTokenProvider, forceRefresh = false) {
    return () => authDataProvider(ctx, refreshTokenProvider, forceRefresh);
}
export function setAuthData(tenant, newAuthData, refreshTokenStorageHandler) {
    _authData = newAuthData;
    refreshTokenStorageHandler.setRefreshToken(tenant, _authData.refresh_token);
}
export function clearAuthData(tenant, refreshTokenStorageHandler) {
    refreshTokenStorageHandler.clearRefreshToken(tenant);
    _authData = null;
}
export function getLoggedUserData(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios({
            method: 'get',
            url: `${baseURL}/auth-api/api/me`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function updateUserData(baseURL, token, userData) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios
            .post(`${baseURL}/auth-api/api/me`, userData, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}
export function deleteUser(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const { login } = yield getLoggedUserData(baseURL, token);
        const response = yield axios
            .request({
            url: `${baseURL}/auth-api/api/me`,
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: { login },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}

==> ordering-core-1.7.0-beta.0.tgz: package/lib/esm/auth.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios, { isAxiosError } from 'axios';
import { handleAPIError } from './apiTools';
import { Mutex } from './utils/mutex';
import { jwtDecode } from 'jwt-decode';
import { EventEmitter } from 'my-events';
const _authDataMap = new Map();
const mutex = new Mutex(250);
const invalidRefreshTokens = new Map();
class TokenExpirationEmitter extends EventEmitter {
    constructor() {
        super();
    }
    notify(tenant, error) {
        this.emit(`refreshTokenInvalid-${tenant}`, error);
    }
}
const RefreshTokenNotifier = new TokenExpirationEmitter();
export function purgeTokenCache() {
    const keys = invalidRefreshTokens.keys();
    for (const key of keys) {
        invalidRefreshTokens.delete(key);
    }
}
export function authorizeWithUserPass(baseUrl, tenant, basicAuthPass, username, password) {
    return __awaiter(this, void 0, void 0, function* () {
        let response = null;
        try {
            response = yield axios.post(`${baseUrl}/auth-oauth2/oauth/token`, { username, password, grant_type: 'password', scope: 'read' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authDataMap.set(tenant, response.data);
            return true;
        }
        catch (error) {
            //console.error('Authorization error');
            //console.error(error);
            return false;
        }
    });
}
export function authorizeWithRefreshToken(baseUrl, tenant, basicAuthPass, refreshToken) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c;
        let response = null;
        try {
            response = yield axios.post(`${baseUrl}/auth-oauth2/oauth/token`, { refresh_token: refreshToken, grant_type: 'refresh_token' }, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                    Authorization: `Basic ${basicAuthPass}`,
                    'X-Tenant': tenant,
                },
            });
            _authDataMap.set(tenant, response.data);
            // console.log('authData', _authData);
            //console.log('REFRESH TOKEN AUTH RESULT ***');
            //console.log(_authData);
            return true;
        }
        catch (error) {
            if (isAxiosError(error) && /^4\d{2}$/.test(`${(_a = error.response) === null || _a === void 0 ? void 0 : _a.status}`)) {
                if (((_c = (_b = error.response) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.error) === 'invalid_grant') {
                    console.warn('invalid_grant', tenant);
                    RefreshTokenNotifier.notify(tenant, error);
                }
                invalidRefreshTokens.set(refreshToken, Date.now());
            }
            //console.error('Authorization error');
            // console.error(error);
            return false;
        }
    });
}
export function isTokenExpired(token) {
    // check if we can use current token
    if (token) {
        try {
            return (
            // 10s before actual expiry
            jwtDecode(token).exp * 1000 < new Date().getTime() + 10000);
        }
        catch (error) {
            // Not a JWT token
            return true;
        }
    }
    return true;
}
function isJwtToken(token) {
    try {
        jwtDecode(token);
        return true;
    }
    catch (error) {
        return false;
    }
}
export const authDataProvider = (ctx_1, refreshTokenStorageHandler_1, ...args_1) => __awaiter(void 0, [ctx_1, refreshTokenStorageHandler_1, ...args_1], void 0, function* (ctx, refreshTokenStorageHandler, forceRefresh = false) {
    // console.log( `authDataProvider invoked ----     currentToken=${  _authData ? _authData.access_token : '(null)'   }`, );
    let authData = _authDataMap.get(ctx.TENANT);
    if (!forceRefresh && authData && authData.access_token) {
        if (!isTokenExpired(authData.access_token)) {
            return { token: authData.access_token, UUID: authData.UUID };
        }
    }
    //console.log('******** GET AUTH TOKEN (REMOTE CALL) ********');
    let loginSuccess = false;
    const refreshToken = yield refreshTokenStorageHandler.getRefreshToken(ctx.TENANT);
    if (refreshToken &&
        (!isJwtToken(refreshToken) || !isTokenExpired(refreshToken))) {
        //console.log('*** AUTHORIZE WITH REFRESH TOKEN ');
        // use mutex to enforce max 1 api call at a time
        loginSuccess = yield mutex.getLock(() => {
            authData = _authDataMap.get(ctx.TENANT);
            if (authData && !isTokenExpired(authData.access_token)) {
                return Promise.resolve(true);
            }
            // If same refresh token was invalid less then 30s ago skip api call
            const invalidTokenTimestamp = invalidRefreshTokens.get(refreshToken);
            if (invalidTokenTimestamp && Date.now() < invalidTokenTimestamp + 30000) {
                return Promise.resolve(false);
            }
            return authorizeWithRefreshToken(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, refreshToken);
        });
    }
    if (!loginSuccess && ctx.anonymousAuth) {
        // console.log('**** ANONYMOUS AUTH ****');
        loginSuccess = yield authorizeWithUserPass(ctx.BASE_URL, ctx.TENANT, ctx.BASIC_AUTH, 'anonymous', '');
    }
    authData = _authDataMap.get(ctx.TENANT);
    if (!loginSuccess || !authData) {
        refreshTokenStorageHandler.clearRefreshToken(ctx.TENANT);
        return { token: '', UUID: '' };
    }
    else {
        refreshTokenStorageHandler.setRefreshToken(ctx.TENANT, authData.refresh_token);
        return { token: authData.access_token, UUID: authData.UUID };
    }
});
export function createAuthDataProvider(ctx, refreshTokenProvider, forceRefresh = false, refreshTokenErrorCallback) {
    RefreshTokenNotifier.on(`refreshTokenInvalid-${ctx.TENANT}`, (error) => {
        refreshTokenErrorCallback === null || refreshTokenErrorCallback === void 0 ? void 0 : refreshTokenErrorCallback(error);
    });
    return () => authDataProvider(ctx, refreshTokenProvider, forceRefresh);
}
export function setAuthData(tenant, newAuthData, refreshTokenStorageHandler) {
    _authDataMap.set(tenant, newAuthData);
    refreshTokenStorageHandler.setRefreshToken(tenant, newAuthData.refresh_token);
}
export function clearAuthData(tenant, refreshTokenStorageHandler) {
    refreshTokenStorageHandler.clearRefreshToken(tenant);
    _authDataMap.delete(tenant);
}
export function getLoggedUserData(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios({
            method: 'get',
            url: `${baseURL}/auth-api/api/me`,
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        }).catch(handleAPIError);
        return response ? response.data : null;
    });
}
export function updateUserData(baseURL, token, userData) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield axios
            .post(`${baseURL}/auth-api/api/me`, userData, {
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}
export function deleteUser(baseURL, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const { login } = yield getLoggedUserData(baseURL, token);
        const response = yield axios
            .request({
            url: `${baseURL}/auth-api/api/me`,
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            data: { login },
        })
            .catch(handleAPIError);
        return (response && response.data) || false;
    });
}
